;;
;; badkpv.pro:  Utility procedure based on the SPICELIB routine
;; BADKPV.
;;
;; badkpv throws a SPICE error exception if the indicated kernel
;; variable is not present in the kernel pool or is present but
;; does not have the expected attributes.
;;
;;
;; The N0061 version of the SPICE Toolkit will provide
;; a routine
;;
;;    cspice_badkpv
;;
;; which will supersede this procedure.
;;
;;
;; Inputs
;; ======
;;
;;    caller     is the name of the routine calling this routine
;;               to check correctness of kernel pool variables.
;;
;;    name       is the name of a kernel pool variable that the
;;               calling program expects to be present in the
;;               kernel pool.
;;
;;    comp       is the comparison operator to use when comparing
;;               the number of components of the kernel pool 
;;               variable specified by `name' with the integer 
;;               `size'.  If `dim' is the actual size of the 
;;               kernel pool variable then `badkpv' will check 
;;               that the sentence
;;
;;                   dim comp size
;;
;;               is a true statement.  If it is not a true 
;;               statement, an error will be signaled.
;;
;;               Allowed values for `comp' and their meanings are:
;;
;;               '='      DIM EQ SIZE
;;               '<'      DIM LT SIZE
;;               '>'      DIM GT SIZE
;;               '=>'     DIM GE SIZE
;;               '<='     DIM LE SIZE
;;
;;
;;    size       is an integer to compare with the actual
;;               number of components of the kernel pool variable
;;               specified by `name'.
;;
;;    divby      is an integer that is one of the factors of the
;;               actual dimension of the specified kernel pool 
;;               variable.  In other words, it is expected that 
;;               `divby' evenly divides the actual dimension of 
;;               `name'. In those cases in which the factors 
;;               of the dimension of `name' are not important, 
;;               set `divby' to 1 in the calling program.
;;
;;    type       is the expected data type of the kernel pool 
;;               variable.  Recognized values are
;;
;;                 'C' for character type
;;                 'N' for numeric type (integer and double
;;                      precision)
;;
;;               The case of `type' is insignificant.  If the value
;;               of `type' is not one of the 2 values given above
;;               no check for the type of the variable will be
;;               performed.
;;
;;
;;
;;    08-MAR-2006 (NJB)
;;


PRO badkpv, caller, name, comp, size, divby, type 

   ;;
   ;; Initialize the traceback message.  We'll use this message
   ;; only if an error is detected, but it's simplest to 
   ;; factor this line of code out of the error handling logic.
   ;;

   trace_msg = '[' + caller + '->badkpv]'


   ;;
   ;; Look up the attributes of this variable in the kernel pool.
   ;;

   cspice_dtpool, name, found, dim, class

   if  ~found  then begin
    
      short_msg = 'SPICE(VARIABLENOTFOUND)'
    
      long_msg  = 'The kernel pool variable ' + name + $
                  ' is not currently present in the '   + $
                  'kernel pool. Possible reasons are '  + $
                  'that the appropriate text kernel '   + $
                  'file has not been loaded via a '     + $
                  'call to cspice_furnsh or that the '  + $
                  'routine cspice_clpool has been '     + $
                  'called after loading the '           + $
                  'appropriate file. '

      MESSAGE, short_msg + ': ' + trace_msg + ' ' + long_msg

      return

   endif



   ;; 
   ;; Compare the dimension of the specified variable with the
   ;; input `size'.
   ;; 

   case comp of

      '=':   ok = ( dim EQ size )

      '<':   ok = ( dim LT size )

      '>':   ok = ( dim GT size )

      '<=':  ok = ( dim LE size )

      '=>':  ok = ( dim GE size )

      else:  begin

         short_msg = 'SPICE(UNKNOWNCOMPARE)'

         long_msg  = 'The comparison operator ' + comp   + $
                     ' is not a recognized value.  The ' + $
                     'recognized values are "<", "<=", ' + $
                     '"=", "=>", ">".'

         MESSAGE, short_msg + ': ' + trace_msg + $
                  ' ' + long_msg

         return

      end

   endcase

   
   ;;
   ;; If the comparison condition was not satisfied, raise 
   ;; an exception and return.
   ;;

   if  ~ok  then begin

      short_msg = 'SPICE(BADVARIABLESIZE)'

      dim_str   = strcompress( string( dim ),   /REMOVE_ALL )
      size_str  = strcompress( string( size ),  /REMOVE_ALL )

      long_msg  = 'The kernel pool variable "' + name + '" is '+ $
                  'expected to have a number of components '   + $
                  'DIM such that the comparison DIM ' + comp   + $
                  ' ' + size_str + ' is true.  However the '   + $
                  'current number of components of "' + name   + $
                  '" is ' + dim_str + '.'

      MESSAGE, short_msg + ': ' + trace_msg + $
               ' ' + long_msg

      return

   endif


   ;; 
   ;; Check to see that `divby' evenly divides the dimension of
   ;; the variable.
   ;;

   if  divby NE 0  then begin
  
      ratio = dim /divby

   endif else begin

      ratio = 1

   endelse


   if  divby*ratio NE dim  then begin

      short_msg = 'SPICE(BADVARIABLESIZE)'

      divby_str = strcompress( string( divby ), /REMOVE_ALL )
      dim_str   = strcompress( string( dim ),   /REMOVE_ALL )

      long_msg  = 'The number of components of the kernel '    + $
                  'pool variable "' + name + '" is required '  + $
                  'to be divisible by ' + divby_str            + $
                  '. However the actual number of components ' + $
                  'is ' + dim_str + ', which is not divisible '+ $ 
                  'by ' + divby_str + '.'

      MESSAGE, short_msg + ': ' + trace_msg + $
               ' ' + long_msg

      return

   endif


   ;;
   ;; Finally check the type of the variable.
   ;;

   if   strcmp(type, 'C', /FOLD_CASE)  then begin

      if  class NE 'C'  then begin

         short_msg = 'SPICE(BADVARIABLETYPE)'

         long_msg  = 'The kernel pool variable "' + name        + $
                     '" must be of type "CHARACTER". However, ' + $
                     'the type is "NUMERIC".'

         MESSAGE, short_msg + ': ' + trace_msg + $
                  ' ' + long_msg

         return

      endif


   endif else if  strcmp(type, 'N', /FOLD_CASE)  then begin

      if  class NE 'N'  then begin

         short_msg = 'SPICE(BADVARIABLETYPE)'

         long_msg  = 'The kernel pool variable "' + name      + $
                     '" must be of type "NUMERIC". However, ' + $
                     'the type is "CHARACTER".'

         MESSAGE, short_msg + ': ' + trace_msg + $
                  ' ' + long_msg

         return


      endif

   endif else begin

      short_msg = 'SPICE(UNKNOWNDATATYPE)'

      long_msg  = 'The data type specifier ' + type   + $
                  ' is not a recognized value.  The ' + $
                  'recognized values are "C" and "N".'

      MESSAGE, short_msg + ': ' + trace_msg + $
               ' ' + long_msg

      return

   endelse


END
