PRO fndevt,  etbeg, etend, step, statef, xset

   ;;
   ;; Find locations of state changes for a boolean function.
   ;;
   ;;   07-MAR-2006 (NJB)
   ;;

   ;;
   ;; Local parameters
   ;;

   CNVLIM = 1.d-6
   MAXITR = 100

   ;;
   ;; Set the cardinality of the input cell to zero.
   ;;
   
   cspice_scard,  0, xset

   ;;
   ;; If the input interval is empty, we're done.
   ;;
   if  etend LE etbeg  then return


   ;;
   ;; The step size must be at least as large as the convergence
   ;; limit.
   ;;

   if  step LE CNVLIM  then begin

      print, 'STEP must be > the convergence limit ', CNVLIM, $
             ' actual value was ', STEP, '.'
      return

   endif

   ;;
   ;; Obtain the initial state; save this as the "lower" state.
   ;;

   lstate = call_function ( statef, etbeg )

   ;;
   ;; Step along the input interval, looking for state changes.
   ;;

   lpt    =  etbeg
   upt    =  cspice_brcktd ( etbeg + step,  etbeg,  etend )


   while  lpt LT etend  do begin

      ;;
      ;; Find the state at the epoch `upt'.  First, check that `upt'
      ;; is actually greater than `lpt'.
      ;;

      if  upt LE lpt  then begin

         ;;
         ;; We're not getting anywhere; the step is too small.
         ;;

         print, 'Upper bracketing epoch UPT = ', UPT,    $
                '; lower epoch LPT = ', LPT, '. This ' + $
                'condition arises when the step size ' + $
                'is too small.'

         return

      endif

      ;;
      ;; Find the state at the upper bound of our step interval.
      ;;

      ustate = call_function ( statef, upt )

      if  ustate NE lstate  then begin

         ;;
         ;; There's a state change between the right endpoint `upt'
         ;; and the left endpoint `lpt'.  Do a binary search to
         ;; locate the epoch at which the state changes.  Note:
         ;; uniqueness of the root is not guaranteed; it's up
         ;; to the caller to choose `step' small enough to ensure
         ;; that only one root can occur in a time interval of
         ;; length `step'.
         ;;

         lower  =  cspice_brcktd ( lpt, etbeg, etend )
         upper  =  cspice_brcktd ( upt, etbeg, etend )
         midpt  =  ( lower + upper ) / 2
         mstate =  call_function ( statef, midpt )

         nitr   =  0
         delta  =  upper - lower


         while ( delta GT CNVLIM ) && ( nitr LE MAXITR ) do begin
 
            ;;
            ;; Adjust our search interval so the length is reduced
            ;; by a factor of two and so that the location of the
            ;; state change remains between LOWER and UPPER.
            ;;

            if  mstate EQ ustate  then begin

               ;;
               ;; The state is the same at the midpoint and the
               ;; upper bound, so a state change must occur
               ;; between the lower bound and the midpoint.
               ;; Make the midpoint the new upper bound.
               ;;

               upper = midpt

            endif else begin

               ;;
               ;; There is a state change between the midpoint
               ;; and the upper bound; the state at the midpoint
               ;; matches the state at the lower bound.  Make
               ;; the midpoint the new lower bound.
               ;;

               lower = midpt

            endelse


            midpt  = ( lower + upper ) / 2
            mstate = call_function ( statef, midpt )

            delta  = upper - lower
            ++ nitr

            ;;
            ;; At this point, the state at `lower' matches the state
            ;; at `lpt' (lstate), and the state at `upper' matches 
            ;; the state at `upt' (ustate).
            ;;

         endwhile

         ;;
         ;; If we dropped out of the loop because we hit the
         ;; iteration limit, we have a problem.
         ;;

         if  delta GT CNVLIM  then begin

            print, 'Binary search failed to converge.'
            print, 'etbeg = ', etbeg
            print, 'lower = ', lower
            print, 'midpt = ', midpt
            print, 'upper = ', upper
            print, 'etend = ', etend

            return

         endif

         ;; 
         ;; The epoch of the state transition has been determined
         ;; to within `cnvlim'.  We'll use `upper' as the epoch of
         ;; the transition.  This ensures that the state at the
         ;; transition epoch is `ustate'.
         ;;
         appndd, upper, xset

         ;; 
         ;; `upper' becomes the left endpoint of the next interval.
         ;; The state at the right interval endpoint `upt' becomes
         ;; the state at the left endpoint of the next interval.
         ;; Calculate `upt' at the next step.  We look up
         ;; `ustate' at the top of the loop.
         ;;

         lpt     =  upper
         lstate  =  ustate
         upt     =  cspice_brcktd ( lpt + step,  etbeg,  etend )

      endif else begin

         ;;
         ;; No state change was found on this step.  `upt' becomes
         ;; the lower bound of the next search interval.  `lstate'
         ;; remains unchanged.  We look up `ustate' at the top of
         ;; the loop.
         ;;

         lpt =  upt
         upt =  cspice_brcktd ( lpt + step,  etbeg,  etend )

      endelse

   endwhile

   ;;
   ;; The left endpoint equals the right endpoint of our search
   ;; interval, so there are no more state changes to be found.
   ;;

END
