PRO makwin,  etbeg, etend, fstate, evtset, evtwin 

   ;; 
   ;; Utility subroutine for creating a window representing
   ;; time intervals when a specified binary state function
   ;; returns "true."
   ;; 
   ;; 
   ;; Subroutine arguments:
   ;; 
   ;;    etbeg         is the start time of a time interval on
   ;;                  which the binary state function
   ;;                  `fstate' is defined.
   ;; 
   ;;    etend         is the end time corresponding to `etbeg'.
   ;; 
   ;;    fstate        is a binary, state function of ET that
   ;;                  returns a boolean value.
   ;; 
   ;;    evtset        is an Icy set containing epochs of
   ;;                  state transitions.  The members `of evtset'
   ;;                  must lie in the interval
   ;; 
   ;;                     [etbeg, etend]
   ;; 
   ;;    evtwin        is an Icy window constructed by this
   ;;                  routine.  `evtwin' contains subintervals
   ;;                  of the interval
   ;; 
   ;;                     [etbeg, etend]
   ;; 
   ;;                  on which fstate(et) is true.  `evtwin'
   ;;                  must be initialized by the caller of this
   ;;                  routine.
   ;; 
   ;; 


   ;; 
   ;; Empty the output window.
   ;; 

   cspice_scard, 0, evtwin

   ;; 
   ;; Get a local copy of the cardinality of the event set.
   ;; 

   card = cspice_card( evtset )

   ;; 
   ;; Let `start_index' be the array start index of
   ;; the data in the set member array evtset.base.
   ;;

   start_index = evtset.data

   ;; 
   ;; If `fstate' is true at `etbeg', then the first state 
   ;; transition is the right endpoint of the first window 
   ;; interval, and every subsequent even-numbered transition marks 
   ;; the start of a window interval. Otherwise, every odd-numbered 
   ;; transition marks the start of a window interval.
   ;; 
      
   if  call_function( fstate, etbeg )  then begin

      ;; 
      ;; The first window interval starts at `etbeg'.
      ;; 

      if  card EQ 0  then begin

         ;; 
         ;; The function is true the whole time.  The
         ;; window consists of the single interval
         ;; 
         ;;    [etbeg, etend]
         ;;

         cspice_wninsd, etbeg, etend, evtwin


      endif else begin

         ;; 
         ;; Insert the first interval into the window.
         ;; 

         cspice_wninsd, etbeg, evtset.base[start_index], evtwin

         ;; 
         ;; Insert into our window any remaining intervals bounded
         ;; by transitions.
         ;; 

         i = 1

         while  (i+1) LE (card-1)  do begin

            j = start_index + i

            cspice_wninsd, evtset.base[j], evtset.base[j+1], evtwin

            i = i + 2

         endwhile


         if  i EQ (card-1)  then begin

            ;; 
            ;; The last interval begins at 
            ;; evtset.base[start_index+card-1]
            ;; and ends at `etend'.
            ;; 
         
            j = start_index + card - 1

            cspice_wninsd,  evtset.base[j],  etend,  evtwin

         endif

      endelse

      ;; 
      ;; We've handled the case where fstate is true at `etbeg'.
      ;; 
      
   endif else begin

      ;; 
      ;; The first interval starts at evtset.base[start_index].
      ;; 
         
      i = 0

      while  (i+1) LE (card-1)  do begin

         j  =  start_index + i

         cspice_wninsd,  evtset.base[j], evtset.base[j+1], evtwin

         i  =  i + 2

      endwhile


      if  i EQ (card-1)  then begin

         ;; 
         ;; The last interval begins at 
         ;; evtset.base[start_index+card-1]
         ;; and ends at `etend'.
         ;; 
         
         j = start_index + card - 1

         cspice_wninsd, evtset.base[j], etend, evtwin

      endif

   ;; 
   ;; We've handled the case where `fstate' is false at `etbeg'.
   ;; 

   endelse

   ;; 
   ;; `evtwin' is ready for use.
   ;; 

END
